<?php
/**
* CWVPSB_Cache_Disk
*
* @since 1.0.0
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

final class CWVPSB_Cache_Disk {

	const FILE_HTML = 'index.html';
	const FILE_GZIP = 'index.html.gz';

	public static function is_permalink() {
		if(is_multisite() && is_plugin_active_for_network(CWVPSB_BASE)){
			return get_site_option('permalink_structure');
		}
		else{
			return get_option('permalink_structure');
		}
		
	}

	public static function store_asset($data) {

		// check if empty
		if ( empty($data) ) {
			wp_die('Asset is empty.');
		}

		// save asset
		self::_create_files(
			$data
		);

	}

	public static function check_asset() {
		return is_readable(
			self::_file_html()
		);
	}

	public static function delete_asset($url) {

		// check if url empty
		if ( empty($url) ) {
			wp_die('URL is empty.');
		}

		// delete
		self::_clear_dir(
			self::_file_path($url)
		);
	}

	public static function clear_cache() {
		self::_clear_dir(
			CWVPSB_CACHE_DIR
		);
	}

	public static function clear_home() {
		
		if ( is_multisite() && is_plugin_active_for_network(CWVPSB_BASE) ) {
			$cwvps_siteurl=get_option('siteurl');
		}
		else{
			$cwvps_siteurl=get_site_option('siteurl');
		}
		$path = sprintf(
			'%s%s%s%s',
			CWVPSB_CACHE_DIR,
			DIRECTORY_SEPARATOR,
			preg_replace('#^https?://#', '', $cwvps_siteurl),
			DIRECTORY_SEPARATOR
		);

		@unlink($path.self::FILE_HTML);
		@unlink($path.self::FILE_GZIP);
	}

	public static function get_asset($data) {
		// set cache handler header
		header('X-Cache-Handler: php');

		// get if-modified request headers
		if ( function_exists( 'apache_request_headers' ) ) {
			$headers = apache_request_headers();
			$http_if_modified_since = ( isset( $headers[ 'If-Modified-Since' ] ) ) ? $headers[ 'If-Modified-Since' ] : '';
			$http_accept = ( isset( $headers[ 'Accept' ] ) ) ? $headers[ 'Accept' ] : '';
			$http_accept_encoding = ( isset( $headers[ 'Accept-Encoding' ] ) ) ? $headers[ 'Accept-Encoding' ] : '';
		} else {
			$http_if_modified_since = ( isset( $_SERVER[ 'HTTP_IF_MODIFIED_SINCE' ] ) ) ? $_SERVER[ 'HTTP_IF_MODIFIED_SINCE' ] : '';
			$http_accept = ( isset( $_SERVER[ 'HTTP_ACCEPT' ] ) ) ? $_SERVER[ 'HTTP_ACCEPT' ] : '';
			$http_accept_encoding = ( isset( $_SERVER[ 'HTTP_ACCEPT_ENCODING' ] ) ) ? $_SERVER[ 'HTTP_ACCEPT_ENCODING' ] : '';
		}

		// check modified since with cached file and return 304 if no difference
		if ( $http_if_modified_since && ( strtotime( $http_if_modified_since ) == filemtime( self::_file_html() ) ) ) {
			header( $_SERVER['SERVER_PROTOCOL'] . ' 304 Not Modified', true, 304 );
			return $data;
			exit;
		}
		// check encoding and deliver gzip file if support
		if ( $http_accept_encoding && ( strpos($http_accept_encoding, 'gzip') !== false ) && is_readable( self::_file_gzip() )  ) {
			header('Content-Encoding: gzip');
			return file_get_contents( self::_file_gzip() );
			exit;
		}

		// deliver cached file (default)
		return file_get_contents( self::_file_html() );
		exit;
	}
	
	public static function get_asset_readfile() {
		// set cache handler header
		header('X-Cache-Handler: php');

		// get if-modified request headers
		if ( function_exists( 'apache_request_headers' ) ) {
			$headers = apache_request_headers();
			$http_if_modified_since = ( isset( $headers[ 'If-Modified-Since' ] ) ) ? $headers[ 'If-Modified-Since' ] : '';
			$http_accept = ( isset( $headers[ 'Accept' ] ) ) ? $headers[ 'Accept' ] : '';
			$http_accept_encoding = ( isset( $headers[ 'Accept-Encoding' ] ) ) ? $headers[ 'Accept-Encoding' ] : '';
		} else {
			$http_if_modified_since = ( isset( $_SERVER[ 'HTTP_IF_MODIFIED_SINCE' ] ) ) ? $_SERVER[ 'HTTP_IF_MODIFIED_SINCE' ] : '';
			$http_accept = ( isset( $_SERVER[ 'HTTP_ACCEPT' ] ) ) ? $_SERVER[ 'HTTP_ACCEPT' ] : '';
			$http_accept_encoding = ( isset( $_SERVER[ 'HTTP_ACCEPT_ENCODING' ] ) ) ? $_SERVER[ 'HTTP_ACCEPT_ENCODING' ] : '';
		}

		// check modified since with cached file and return 304 if no difference
		if ( $http_if_modified_since && ( strtotime( $http_if_modified_since ) == filemtime( self::_file_html() ) ) ) {
			header( $_SERVER['SERVER_PROTOCOL'] . ' 304 Not Modified', true, 304 );
			exit;
		}
		// check encoding and deliver gzip file if support
		if ( $http_accept_encoding && ( strpos($http_accept_encoding, 'gzip') !== false ) && is_readable( self::_file_gzip() )  ) {
			header('Content-Encoding: gzip');
			readfile( self::_file_gzip() );
			exit;
		}

		// deliver cached file (default)
		readfile( self::_file_html() );
		exit;
	}

	private static function _cache_signatur() {
		return sprintf(
			"\n\n<!-- %s @ %s",
			'Cached Copy, Generated By Core Web Vitals & PageSpeed Booster',
			date_i18n(
				'd.m.Y H:i:s',
				current_time('timestamp')
			)
		);
	}

	private static function _create_files($data) {

		// create folder
		if ( ! wp_mkdir_p( self::_file_path() ) ) {
			wp_die('Unable to create directory.');
		}

		// get base signature
		$cache_signature = self::_cache_signatur();

		// create files
		self::_create_file( self::_file_html(), $data.$cache_signature." -->" ); 
	}

	private static function _create_file($file, $data) {

		// open file handler
		if ( ! $handle = @fopen($file, 'wb') ) {
			wp_die('Can not write to file.');
		}

		// write
		@fwrite($handle, $data);
		fclose($handle);
		clearstatcache();

		// set permissions
		$stat = @stat( dirname($file) );
		$perms = $stat['mode'] & 0007777;
		$perms = $perms & 0000666;
		@chmod($file, $perms);
		clearstatcache();
	}

	private static function _clear_dir($dir) {

		// remove slashes
		$dir = untrailingslashit($dir);

		// check if dir
		if ( ! is_dir($dir) ) {
			return;
		}

		// get dir data
		$objects = array_diff(
			scandir($dir),
			array('..', '.')
		);

		if ( empty($objects) ) {
			return;
		}

		foreach ( $objects as $object ) {
			// full path
			$object = $dir. DIRECTORY_SEPARATOR .$object;

			// check if directory
			if ( is_dir($object) ) {
				self::_clear_dir($object);
			} else {
				unlink($object);
			}
		}

		// delete
		@rmdir($dir);

		// clears file status cache
		clearstatcache();
	}

	private static function _file_path($path = NULL) {

		$path = sprintf(
			'%s%s%s%s',
			CWVPSB_CACHE_DIR,
			DIRECTORY_SEPARATOR,
			parse_url(
				'http://' .strtolower($_SERVER['HTTP_HOST']),
				PHP_URL_HOST
			),
			parse_url(
				( $path ? $path : $_SERVER['REQUEST_URI'] ),
				PHP_URL_PATH
			)
		);

		if ( is_file($path) > 0 ) {
			wp_die('Path is not valid.');
		}

		return trailingslashit($path);
	}

	private static function _file_html() {
		return self::_file_path(). self::FILE_HTML;
	}

	private static function _file_gzip() {
		return self::_file_path(). self::FILE_GZIP;
	}
}